import Serial from "./serial.js"

export default class UsbNfcReader {
    constructor() {
        this.textEncoder = new TextEncoder();
        this.locked = false;
        //Tentative de connexion/reconnexion automatique
        Serial.getPorts().then(ports => {
            if (ports.length == 0) {
                console.log('No devices found.');
            } else {
                this.port = ports[0];
                console.log("Port choisi :", this.port);
                this.connect();
            }
        this.locked = false;
        });
    }

    selectAndConnect() {
        Serial.requestPort().then(selectedPort => {
            this.port = selectedPort;
            this.connect();
          }).catch(error => {
            console.log('Connection error: ' + error);
          });
    }

    disconnect() {
        this.port.disconnect();
        this.port = null;
    }

    connect() {
        console.log('Connecting to ' + this.port.device_.productName + '...');
        this.port.connect().then(() => {
          console.log(this.port, 'Connected.');

          //Important : callback de réception
          this.port.onReceive = data => {
            let textDecoder = new TextDecoder();
            this.receivedMsg = textDecoder.decode(data);
            console.log("Reçu :" + this.receivedMsg);
          }
          this.port.onReceiveError = error => {
            console.log('Receive error: ' + error);
          };
          return true;
        }, error => {
          console.log('Connection error: ' + error);
          return false;
        });
    }

    readTagUid() {
      return new Promise((resolve, reject) => {
        this.locked = true;
        this.sendString("GETID_")
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msg = textDecoder.decode(data);
          msg = msg.replace(/(\r\n|\n|\r)/gm, "");
          console.log("Reçu :" + msg);
          let cmdIdentifier = msg.split('_')[0]
          if (msg == "RECEIVED_GETID;") {
            console.log("GETID request ACK by Arduino")
          } else if (cmdIdentifier == "UID" && msg.slice(-1) == ';') {
            this.locked = false;
            resolve(msg.split('_')[1].slice(0, -1))        
          } else {
            reject("Bad message structure received")
          }
          
        }
      })
    }

    readCard() {
      return new Promise((resolve, reject) => {
        this.locked = true;
        this.sendString("READ_")
        var failCount = 0
        var msg = ""
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msgBuff = textDecoder.decode(data);
          msgBuff = msgBuff.replace(/(\r\n|\n|\r)/gm, "").replace(/[^\w;]+/g, '');
          msg += msgBuff
          if (msg.includes("CONTENT_BEGIN_en") && msg.includes("_END")) {
            msg = msg.split("CONTENT_BEGIN_en")
            msg = msg[msg.length-1].split("_END")[0]
            this.locked = false;
            resolve(msg)
          } else if (!msg.includes("RECEIVED_READ")) {
            if (failCount > 100) {
              reject("Too many reading fail")
            }
            failCount++
          }
        }
      })
    }

    writeCard(content) {
      return new Promise((resolve, reject) => {
        this.locked = true;
        this.sendString("WRITE_" + content)
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msg = textDecoder.decode(data);
          msg = msg.replace(/(\r\n|\n|\r)/gm, "");
          console.log("recu ", msg)
          if (msg == "WRITTEN;") {
            console.log("wrote team message")
            this.locked = false;
            resolve("")
          } else if (!msg.includes("RECEIVED_WRITE")) {
            reject("Write : Bad message structure received")
          }
        }
      })
    }

    cleanCard() {
      return new Promise((resolve, reject) => {
        this.locked = true;
        this.sendString("CLEAN_")
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msg = textDecoder.decode(data);
          msg = msg.replace(/(\r\n|\n|\r)/gm, "");
          console.log("recu ", msg)
          if (msg == "CLEANED;") {
            console.log("Cleaned card")
            this.locked = false;
            resolve("")
          } else if (!msg.includes("RECEIVED_CLEAN")) {
            reject("Clean : Bad message structure received")
          }
        }
      })
    }

    formatCard() {
      return new Promise((resolve, reject) => {
        this.locked = true;
        this.sendString("FORMAT_")
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msg = textDecoder.decode(data);
          msg = msg.replace(/(\r\n|\n|\r)/gm, "");
          console.log("recu ", msg)
          if (msg == "FORMATED;") {
            console.log("Formated card")
            this.locked = false;
            resolve("")
          } else if (!msg.includes("RECEIVED_FORMAT")) {
            reject("Format : Bad message structure received")
          }
        }
      })
    }

    eraseCard() {
      return new Promise((resolve, reject) => {
        this.locked = true;
        this.sendString("ERASE_")
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msg = textDecoder.decode(data);
          msg = msg.replace(/(\r\n|\n|\r)/gm, "");
          console.log("recu ", msg)
          if (msg == "ERASED;") {
            console.log("Erased card")
            this.locked = false;
            resolve("")
          } else if (!msg.includes("RECEIVED_ERASE")) {
            reject("Erase : Bad message structure received")
          }
        }
      })
    }

    ping() {
      return new Promise((resolve, reject) => {
        if (this.locked) {
          console.log("Trying to ping while performing another task")
          return resolve("")
        }
        this.sendString("PING_")
        var timeout = setTimeout(() => {
          reject("ping timeout")}
          
          , 2000);
        this.port.onReceive = data => {
          let textDecoder = new TextDecoder();
          let msg = textDecoder.decode(data);
          msg = msg.replace(/(\r\n|\n|\r)/gm, "");
          if (msg == "PONG;") {
            clearTimeout(timeout)
            resolve("")
          } 
        }
      })
    }
    //Envoie ce string à l'Arduino
    sendString(str) {
        if (this.port !== undefined) {
            this.port.send(this.textEncoder.encode(str)).catch(error => {
                console.log('Send error: ' + error);
            });
        }
    }
}
