const express = require('express');
const bodyParser = require('body-parser');
const path = require('path');
const cors = require('cors');
const mongoose = require('mongoose');



const config = require('./db.js');
const teamRoute = require('./db/team.route')
const passport = require('passport');
const LocalStrategy = require('passport-local').Strategy;
const connectEnsureLogin = require('connect-ensure-login');
const port = 80;



//Configuration MongoDB
mongoose.Promise = global.Promise;
console.log("Trying to connect to DB :" +config.DB)
mongoose.connect(config.DB, {useNewUrlParser: true, useUnifiedTopology: true}).then(
  () => {console.log('Database connected')},
  err => {console.log("Database connection failed :", err)}
);


//Configuration cors et bodyparser
const app = express();
app.use(cors({credentials: true, origin: 'http://localhost:8080'}));
app.use(bodyParser.urlencoded({extended: true}));
app.use(bodyParser.json());

const expressSession = require('express-session')({
  secret: 'secret',
  resave: false,
  saveUninitialized: false
});
app.use(expressSession);
app.use(passport.initialize());
app.use(passport.session());


const UserDetails = require('./db/login.model')
passport.use(UserDetails.createStrategy())
passport.serializeUser(UserDetails.serializeUser())
passport.deserializeUser(UserDetails.deserializeUser())


const history = require('connect-history-api-fallback');
const staticFileMiddleware = express.static(path.join(__dirname + '/dist'));

app.use(staticFileMiddleware);
app.use(history({
  disableDotRule: true,
  verbose: true
}));
app.use(staticFileMiddleware);

app.get('/', function (req, res) {
  res.render(path.join(__dirname + '/dist/index.html'));
});

//Appel aux routeurs
app.use('/school', teamRoute);

app.post('/login',
  passport.authenticate('local'), function(req, res) {
    res.send(req.user)
    console.log("Login de", req.user.username)
  } 
);

app.post('/logout', function(req, res){
  if (req.user) {
    console.log("Logout de :", req.user.username)
    req.logout();
    res.send("logout ok")
  }
});

app.post('/register', function(req, res) {
  if (req.body.username && req.body.password && req.body.schoolName) {
    UserDetails.register({username: req.body.username, schoolName: req.body.schoolName, active: false}, req.body.password);
    console.log("Register de", req.body.username);
    res.send("register ok")
  }
  
})

app.post('/loginCheck', function(req, res) {
  if (req.user) {
    res.send('ok')
  } else {
    res.send('failed')
  }
})

//Initialisation
app.listen(port, () => {
  console.log(`Example app listening at http://localhost:${port}`)
});

//UserDetails.register({username:'thomas', schoolName:"INSA", active: false}, 'thomas');